@@ -1,3 +1,5 @@
+from __future__ import annotations
+
 """
 Implements the one shot version of SCML
 """
@@ -11,14 +13,7 @@ from collections import defaultdict
 from typing import Any
 from typing import Callable
 from typing import Collection
-from typing import Dict
 from typing import Iterable
-from typing import List
-from typing import Optional
-from typing import Set
-from typing import Tuple
-from typing import Type
-from typing import Union
 
 import networkx as nx
 import numpy as np
@@ -99,10 +94,10 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         self,
         # SCML2020 specific parameters
         catalog_prices: np.ndarray,
-        profiles: List[OneShotProfile],
-        agent_types: List[Type["OneShotAgent"]],
-        agent_params: List[Dict[str, Any]],
-        catalog_quantities: Union[int, np.ndarray] = 50,
+        profiles: list[OneShotProfile],
+        agent_types: list[type[OneShotAgent]],
+        agent_params: list[dict[str, Any]],
+        catalog_quantities: int | np.ndarray = 50,
         # breach processing parameters
         financial_report_period=5,
         bankruptcy_limit=0.0,
@@ -113,7 +108,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         exogenous_dynamic: bool = False,
         exogenous_force_max: bool = False,
         # factory parameters
-        initial_balance: Union[np.ndarray, Tuple[int, int], int] = 1000,
+        initial_balance: np.ndarray | tuple[int, int] | int = 1000,
         # General SCML2020World Parameters
         compact=False,
         no_logs=False,
@@ -126,19 +121,21 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         neg_hidden_time_limit=3 * 60,
         neg_step_time_limit=60,
         negotiation_speed=0,
-        avoid_ultimatum=True,
+        avoid_ultimatum=False,
+        shuffle_negotiations=False,
         # public information
         publish_exogenous_summary=True,
         publish_trading_prices=True,
         # negotiation params,
         price_multiplier=2.0,
+        wide_price_range=True,
         # trading price parameters
         trading_price_discount=0.9,
         # simulation parameters
         signing_delay=0,
         force_signing=False,
         batch_signing=True,
-        name: str = None,
+        name: str | None = None,
         # debugging parameters
         agent_name_reveals_position: bool = True,
         agent_name_reveals_type: bool = True,
@@ -147,19 +144,20 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         inventory_valuation_trading=0,
         **kwargs,
     ):
-        self._profits: Dict[str, List[float]] = defaultdict(list)
-        self._breach_levels: Dict[str, List[float]] = defaultdict(list)
-        self._breaches_of: Dict[str, List[bool]] = defaultdict(list)
+        self._profits: dict[str, list[float]] = defaultdict(list)
+        self._breach_levels: dict[str, list[float]] = defaultdict(list)
+        self._breaches_of: dict[str, list[bool]] = defaultdict(list)
         self.trading_price_discount = trading_price_discount
         self.catalog_quantities = catalog_quantities
         self.publish_exogenous_summary = publish_exogenous_summary
         self.price_multiplier = price_multiplier
+        self.wide_price_range = wide_price_range
         self.publish_trading_prices = publish_trading_prices
         self.penalize_bankrupt_for_future_contracts = (
             penalize_bankrupt_for_future_contracts
         )
-        self.agent_disposal_cost: Dict[str, List[float]] = dict()
-        self.agent_shortfall_penalty: Dict[str, List[float]] = dict()
+        self.agent_disposal_cost: dict[str, list[float]] = dict()
+        self.agent_shortfall_penalty: dict[str, list[float]] = dict()
         kwargs["log_to_file"] = not no_logs
         if compact:
             kwargs["event_file_name"] = None
@@ -183,6 +181,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             bulletin_board=None,
             breach_processing=BreachProcessing.NONE,
             awi_type="scml.oneshot.OneShotAWI",
+            shuffle_negotiations=shuffle_negotiations,
             mechanisms={
                 "negmas.sao.SAOMechanism": mechanisms.get(
                     "negmas.sao.SAOMechanism",
@@ -277,6 +276,8 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             else -int(0.5 + bankruptcy_limit * initial_balance.mean())
         )
         self.info.update(
+            avoid_ultimatum=avoid_ultimatum,
+            shuffle_negotiations=shuffle_negotiations,
             process_inputs=process_inputs,
             process_outputs=process_outputs,
             catalog_prices=catalog_prices,
@@ -307,6 +308,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             publish_exogenous_summary=publish_exogenous_summary,
             publish_trading_prices=publish_trading_prices,
             selected_price_multiplier=price_multiplier,
+            wide_price_range=wide_price_range,
         )
 
         if not isinstance(agent_types, Iterable):
@@ -325,7 +327,6 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             if financial_report_period >= 1
             else int(0.5 + financial_report_period * n_steps)
         )
-        # breakpoint()
         agent_types = [get_class(_) for _ in agent_types]
         for p in agent_params:
             p["obj"] = get_class(p["controller_type"])(
@@ -417,6 +418,8 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             a = instantiate(atype, **aparams)
             a.id = a.name
             if a.adapted_object:
+                a.adapted_object.id = a.id
+                a.adapted_object.name = a.name
                 if isinstance(a.adapted_object, OneShotAgent):
                     a.adapted_object.connect_to_oneshot_adapter(a)
                 else:
@@ -435,14 +438,14 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
 
         self.agent_n_contracts = dict(zip((_.id for _ in agents), itertools.repeat(0)))
 
-        self.suppliers: List[List[str]] = [[] for _ in range(n_products)]
-        self.consumers: List[List[str]] = [[] for _ in range(n_products)]
-        self.agent_processes: Dict[str, List[int]] = defaultdict(list)
-        self.agent_inputs: Dict[str, List[int]] = defaultdict(list)
-        self.agent_outputs: Dict[str, List[int]] = defaultdict(list)
-        self.agent_consumers: Dict[str, List[str]] = defaultdict(list)
-        self.agent_suppliers: Dict[str, List[str]] = defaultdict(list)
-        self.agent_profiles: Dict[str, OneShotProfile] = defaultdict(list)
+        self.suppliers: list[list[str]] = [[] for _ in range(n_products)]
+        self.consumers: list[list[str]] = [[] for _ in range(n_products)]
+        self.agent_processes: dict[str, list[int]] = defaultdict(list)
+        self.agent_inputs: dict[str, list[int]] = defaultdict(list)
+        self.agent_outputs: dict[str, list[int]] = defaultdict(list)
+        self.agent_consumers: dict[str, list[str]] = defaultdict(list)
+        self.agent_suppliers: dict[str, list[str]] = defaultdict(list)
+        self.agent_profiles: dict[str, OneShotProfile] = defaultdict(list)
 
         self.consumers[n_products - 1].append(SYSTEM_BUYER_ID)
         self.agent_processes[SYSTEM_BUYER_ID] = []
@@ -501,10 +504,10 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             is_system_agent(k) or self.agent_inputs[k][0] == self.agent_outputs[k] - 1
             for k in self.agent_inputs.keys()
         ), f"Some agents have outputs != input+1\n{self.agent_outputs}\n{self.agent_inputs}"
-        self.is_bankrupt: Dict[str, bool] = dict(
+        self.is_bankrupt: dict[str, bool] = dict(
             zip(self.agents.keys(), itertools.repeat(False))
         )
-        self.exogenous_contracts: Dict[int : List[Contract]] = defaultdict(list)
+        self.exogenous_contracts: dict[int : list[Contract]] = defaultdict(list)
         for c in exogenous_contracts:
             seller_id = agents[c.seller].id if c.seller >= 0 else SYSTEM_SELLER_ID
             buyer_id = agents[c.buyer].id if c.buyer >= 0 else SYSTEM_BUYER_ID
@@ -557,10 +560,10 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         self.initial_balances = dict(zip(self.agents.keys(), initial_balance))
         self._max_n_lines = max(_.n_lines for _ in self.profiles)
         self.a2i = dict(zip((_.id for _ in agents), range(n_agents)))
-        self._current_issues: List[List[Issue]] = []
-        self.__contracts: Dict[str, List[Contract]] = defaultdict(list)
+        self._current_issues: list[list[Issue]] = []
+        self.__contracts: dict[str, list[Contract]] = defaultdict(list)
 
-        def values(x: Union[int, Tuple[int, int]]):
+        def values(x: int | tuple[int, int]):
             if not isinstance(x, Iterable):
                 return int(x), int(x)
             return int(x[0]), int(x[1])
@@ -606,43 +609,46 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
     @classmethod
     def generate(
         cls,
-        agent_types: List[Union[str, Type["OneShotAgent"]]],
-        agent_params: List[Dict[str, Any]] = None,
-        n_steps: Union[Tuple[int, int], int] = (50, 200),
-        n_processes: Union[Tuple[int, int], int] = 2,
-        n_lines: Union[np.ndarray, Tuple[int, int], int] = 10,
-        n_agents_per_process: Union[np.ndarray, Tuple[int, int], int] = (4, 8),
-        process_inputs: Union[np.ndarray, Tuple[int, int], int] = 1,
-        process_outputs: Union[np.ndarray, Tuple[int, int], int] = 1,
-        production_costs: Union[np.ndarray, Tuple[int, int], int] = (1, 4),
-        profit_means: Union[np.ndarray, Tuple[float, float], float] = (0.1, 0.2),
-        profit_stddevs: Union[np.ndarray, Tuple[float, float], float] = 0.05,
-        max_productivity: Union[np.ndarray, Tuple[float, float], float] = (0.8, 1.0),
-        initial_balance: Optional[Union[np.ndarray, Tuple[int, int], int]] = None,
+        agent_types: list[str | type[OneShotAgent]],
+        agent_params: list[dict[str, Any]] | None = None,
+        agent_processes: list[int] | None = None,
+        n_steps: tuple[int, int] | int = (50, 200),
+        n_processes: tuple[int, int] | int = 2,
+        n_lines: np.ndarray | tuple[int, int] | int = 10,
+        n_agents_per_process: np.ndarray | tuple[int, int] | int = (4, 8),
+        process_inputs: np.ndarray | tuple[int, int] | int = 1,
+        process_outputs: np.ndarray | tuple[int, int] | int = 1,
+        production_costs: np.ndarray | tuple[int, int] | int = (1, 4),
+        profit_means: np.ndarray | tuple[float, float] | float = (0.1, 0.2),
+        profit_stddevs: np.ndarray | tuple[float, float] | float = 0.05,
+        max_productivity: np.ndarray | tuple[float, float] | float = (0.8, 1.0),
+        initial_balance: np.ndarray | tuple[int, int] | int | None = None,
         cost_increases_with_level=True,
         equal_exogenous_supply=False,
         equal_exogenous_sales=False,
-        exogenous_supply_predictability: Union[Tuple[float, float], float] = (0.6, 0.9),
-        exogenous_sales_predictability: Union[Tuple[float, float], float] = (0.6, 0.9),
-        exogenous_control: Union[Tuple[float, float], float] = -1,
-        cash_availability: Union[Tuple[float, float], float] = (1.5, 2.5),
+        exogenous_supply_predictability: tuple[float, float] | float = (0.6, 0.9),
+        exogenous_sales_predictability: tuple[float, float] | float = (0.6, 0.9),
+        exogenous_control: tuple[float, float] | float = -1,
+        cash_availability: tuple[float, float] | float = (1.5, 2.5),
         force_signing=True,
         profit_basis=np.max,
-        disposal_cost: Union[np.ndarray, Tuple[float, float], float] = (0.0, 0.2),
-        shortfall_penalty: Union[np.ndarray, Tuple[float, float], float] = (0.2, 1.0),
-        disposal_cost_dev: Union[np.ndarray, Tuple[float, float], float] = (0.0, 0.02),
-        shortfall_penalty_dev: Union[np.ndarray, Tuple[float, float], float] = (
+        disposal_cost: np.ndarray | tuple[float, float] | float = (0.0, 0.2),
+        shortfall_penalty: np.ndarray | tuple[float, float] | float = (0.2, 1.0),
+        disposal_cost_dev: np.ndarray | tuple[float, float] | float = (0.0, 0.02),
+        shortfall_penalty_dev: np.ndarray
+        | tuple[float, float]
+        | float = (
             0.0,
             0.1,
         ),
-        exogenous_price_dev: Union[np.ndarray, Tuple[float, float], float] = (0.1, 0.2),
-        price_multiplier: Union[np.ndarray, Tuple[float, float], float] = (1.5, 2.0),
+        exogenous_price_dev: np.ndarray | tuple[float, float] | float = (0.1, 0.2),
+        price_multiplier: np.ndarray | tuple[float, float] | float = (1.5, 2.0),
         random_agent_types: bool = False,
-        penalties_scale: Union[str, List[str]] = "trading",
+        penalties_scale: str | list[str] = "trading",
         cap_exogenous_quantities: bool = True,
         method="profitable",
         **kwargs,
-    ) -> Dict[str, Any]:
+    ) -> dict[str, Any]:
         """
         Generates the configuration for a world
 
@@ -661,6 +667,8 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             max_productivity:  Maximum possible productivity per level (i.e. process).
             initial_balance: The initial balance of all agents
             n_agents_per_process: Number of agents per process
+            agent_processes: The process for each agent. If not `None` , it will override `n_agents_per_process` and must be a list/tuple
+                             of the same length as `agent_types` . Morevoer, `random_agent_types` must be False in this case
             cost_increases_with_level: If true, production cost will be higher for processes nearer to the final
                                        product.
             profit_basis: The statistic used when controlling catalog prices by profit arguments. It can be np.mean,
@@ -700,6 +708,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
                             price in the contract and `non` means the `storage-cost`
                             and `shortfall_penalty` are absolute values (in money unit).
                             If not given will be read through the AWI
+            method: the generation method. This is only for compatibility with SCML2020World and is not used.
             **kwargs:
 
         Returns:
@@ -708,6 +717,11 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
 
         Remarks:
 
+            - There are two general ways to use this generator:
+                1. Pass `random_agent_types = True`, and pass `agent_types`, `agent_processes` to control placement of each
+                   agent in each level of the production graph.
+                2. Pass `random_agent_types = False` and pass `agent_types`, `n_agents_per_process` to make the system randomly
+                   place the specified number of agents in each production level
             - Most parameters (i.e. `process_inputs` , `process_outputs` , `n_agents_per_process` , `costs` ) can
               take a single value, a tuple of two values, or a list of values.
               If it has a single value, it is repeated for all processes/factories as appropriate. If it is a
@@ -716,11 +730,20 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
               it is otherwise, it is used to sample values for each process.
 
         """
+        if agent_processes is not None and random_agent_types:
+            raise ValueError(
+                "You cannot pass `agent_processes` and use `random_agent_types`. The first is only used when you want to fix the assignment of all agents to specific processes which is compatible with randomizing agnet types"
+            )
+        if agent_processes is not None and len(agent_processes) != len(agent_types):
+            raise ValueError(
+                f"Length of `agent_processes` ({len(agent_processes)}) must equal the length of `agent_types` ({len(agent_types)})"
+            )
         info = dict(
             n_steps=n_steps,
             n_processes=n_processes,
             n_lines=n_lines,
             force_signing=force_signing,
+            agent_processes=agent_processes,
             n_agents_per_process=n_agents_per_process,
             process_inputs=process_inputs,
             process_outputs=process_outputs,
@@ -765,7 +788,23 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
 
         process_inputs = make_array(process_inputs, n_processes, dtype=int)
         process_outputs = make_array(process_outputs, n_processes, dtype=int)
-        n_agents_per_process = make_array(n_agents_per_process, n_processes, dtype=int)
+        fixed_assignment = agent_processes is not None and not random_agent_types
+        if agent_processes is not None:
+            pcount = defaultdict(int)
+            for i in agent_processes:
+                pcount[i] += 1
+            pnums = list(pcount.keys())
+            assert (
+                min(pnums) == 0 and max(pnums) == len(pnums) - 1
+            ), f"`agent_processes` is invalid: {agent_processes} as it leads to the following `n_agents_per_process`: {dict(pcount)}"
+            n_agents_per_process = np.asarray([pcount[i] for i in range(len(pnums))])
+            assert not any(
+                _ <= 0 for _ in n_agents_per_process
+            ), f"We have some levels with no processes"
+        else:
+            n_agents_per_process = make_array(
+                n_agents_per_process, n_processes, dtype=int
+            )
         profit_means = make_array(profit_means, n_processes, dtype=float)
         profit_stddevs = make_array(profit_stddevs, n_processes, dtype=float)
         max_productivity = make_array(
@@ -786,7 +825,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             else:
                 assert len(agent_params) == 1
                 agent_params = [copy.copy(agent_params[0]) for _ in range(n_agents)]
-        elif len(agent_types) != n_agents or random_agent_types:
+        elif not fixed_assignment:
             if agent_params is None:
                 agent_params = [dict() for _ in range(len(agent_types))]
             if isinstance(agent_params, dict):
@@ -969,8 +1008,8 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
                 for p in range(n_products)
             ]
         ).astype(int)
-        profile_info: List[
-            Tuple[OneShotProfile, np.ndarray, np.ndarray, np.ndarray, np.ndarray]
+        profile_info: list[
+            tuple[OneShotProfile, np.ndarray, np.ndarray, np.ndarray, np.ndarray]
         ] = []
 
         nxt = 0
@@ -1203,7 +1242,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
     def complete_contract_execution(self, *args, **kwargs):
         pass
 
-    def start_contract_execution(self, contract: Contract) -> Optional[Set[Breach]]:
+    def start_contract_execution(self, contract: Contract) -> set[Breach] | None:
         return set()
 
     def _update_exogenous(self, s):
@@ -1279,6 +1318,12 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             self._output_quantity = defaultdict(int)
             self._output_price = defaultdict(int)
 
+            # Reset all agents
+            # ================
+            for aid, a in self.agents.items():
+                if hasattr(a, "reset"):
+                    a.reset()
+
             # request all negotiations
             # ========================
             self._make_negotiations()
@@ -1288,6 +1333,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             for aid, a in self.agents.items():
                 if hasattr(a, "before_step"):
                     a.before_step()
+
             return
 
         # update trading price information
@@ -1383,7 +1429,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         perpetrator,
         level,
         type_,
-    ) -> Dict[str, Any]:
+    ) -> dict[str, Any]:
         return {
             "perpetrator": perpetrator,
             "perpetrator_name": perpetrator,
@@ -1426,7 +1472,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         contract = self._adjust_contract_types(contract)
         return contract.agreement["quantity"] * contract.agreement["unit_price"]
 
-    def contract_record(self, contract: Contract) -> Dict[str, Any]:
+    def contract_record(self, contract: Contract) -> dict[str, Any]:
         contract = self._adjust_contract_types(contract)
         c = {
             "id": contract.id,
@@ -1453,7 +1499,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         )
         return c
 
-    def breach_record(self, breach: Breach) -> Dict[str, Any]:
+    def breach_record(self, breach: Breach) -> dict[str, Any]:
         return {
             "perpetrator": breach.perpetrator,
             "perpetrator_name": breach.perpetrator,
@@ -1495,7 +1541,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             scores[a.id] for a in self.agents.values() if not is_system_agent(a.id)
         )
 
-    def relative_welfare(self, include_bankrupt: bool = False) -> Optional[float]:
+    def relative_welfare(self, include_bankrupt: bool = False) -> float | None:
         """Total welfare relative to expected value. Returns None if no expectation is found in self.info"""
         if "expected_income" not in self.info.keys():
             return None
@@ -1511,7 +1557,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             and contract.agreement["quantity"] > 0
         )
 
-    def scores(self, assets_multiplier: float = 0.5) -> Dict[str, float]:
+    def scores(self, assets_multiplier: float = 0.0) -> dict[str, float]:
         """Scores of all agents given the asset multiplier.
 
         Args:
@@ -1602,22 +1648,22 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         return contracts
 
     @property
-    def system_agents(self) -> List[_SystemAgent]:
+    def system_agents(self) -> list[_SystemAgent]:
         """Returns the two system agents"""
         return [_ for _ in self.agents.values() if is_system_agent(_.id)]
 
     @property
-    def system_agent_names(self) -> List[str]:
+    def system_agent_names(self) -> list[str]:
         """Returns the names two system agents"""
         return [_ for _ in self.agents.keys() if is_system_agent(_)]
 
     @property
-    def non_system_agents(self) -> List[DefaultOneShotAdapter]:
+    def non_system_agents(self) -> list[DefaultOneShotAdapter]:
         """Returns all agents except system agents"""
         return [_ for _ in self.agents.values() if not is_system_agent(_.id)]
 
     @property
-    def non_system_agent_names(self) -> List[str]:
+    def non_system_agent_names(self) -> list[str]:
         """Returns names of all agents except system agents"""
         return [_ for _ in self.agents.keys() if not is_system_agent(_)]
 
@@ -1633,16 +1679,16 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
 
     def draw(
         self,
-        steps: Optional[Union[Tuple[int, int], int]] = None,
+        steps: tuple[int, int] | int | None = None,
         what: Collection[str] = DEFAULT_EDGE_TYPES,
         who: Callable[[Agent], bool] = None,
-        where: Callable[[Agent], Union[int, Tuple[float, float]]] = None,
+        where: Callable[[Agent], int | tuple[float, float]] = None,
         together: bool = True,
         axs: Collection[Axis] = None,
         ncols: int = 4,
-        figsize: Tuple[int, int] = (15, 15),
+        figsize: tuple[int, int] = (15, 15),
         **kwargs,
-    ) -> Union[Tuple[Axis, nx.Graph], Tuple[List[Axis], List[nx.Graph]]]:
+    ) -> tuple[Axis, nx.Graph] | tuple[list[Axis], list[nx.Graph]]:
         if where is None:
             where = (
                 lambda x: self.n_processes + 1
@@ -1667,12 +1713,13 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         self,
         agent_id: str,
         product: int,
-        quantity: Union[int, Tuple[int, int]],
-        unit_price: Union[int, Tuple[int, int]],
-        time: Union[int, Tuple[int, int]],
-        controller: Optional[SAOController] = None,
-        negotiators: List[SAONegotiator] = None,
-        extra: Dict[str, Any] = None,
+        quantity: int | tuple[int, int],
+        unit_price: int | tuple[int, int],
+        time: int | tuple[int, int],
+        controller: SAOController | None = None,
+        negotiators: list[SAONegotiator] | None = None,
+        extra: dict[str, Any] | None = None,
+        consumer_starts: bool = True,
     ) -> bool:
         """
         Requests negotiations (used internally)
@@ -1688,13 +1735,14 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             negotiators: An optional list of negotiators to use for negotiating with the given partners (in the same
                          order).
             extra: Extra information accessible through the negotiation annotation to the caller
+            consumer_starts: Whether the consumer or supplier sends the first offer in the negotiation
 
         Returns:
 
             `True` if the partner accepted and the negotiation is ready to start
 
         """
-        if self.is_bankrupt[agent_id]:
+        if self.is_bankrupt[agent_id] or is_system_agent(agent_id):
             return True
         if controller is not None and negotiators is not None:
             raise ValueError(
@@ -1706,7 +1754,15 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             )
         if extra is None:
             extra = dict()
-        partners = [_ for _ in self.suppliers[product] if not self.is_bankrupt[_]]
+
+        responding_agents = (
+            self.suppliers[product] if consumer_starts else self.consumers[product]
+        )
+        partners = [
+            _
+            for _ in responding_agents
+            if not self.is_bankrupt[_] and not is_system_agent(_)
+        ]
         if not partners:
             return True
         if negotiators is None:
@@ -1724,6 +1780,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
                 partner,
                 negotiator,
                 extra,
+                consumer_starts=consumer_starts,
             )
             for partner, negotiator in zip(partners, negotiators)
         ]
@@ -1736,12 +1793,13 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         self,
         agent_id: str,
         product: int,
-        quantity: Union[int, Tuple[int, int]],
-        unit_price: Union[int, Tuple[int, int]],
-        time: Union[int, Tuple[int, int]],
+        quantity: int | tuple[int, int],
+        unit_price: int | tuple[int, int],
+        time: int | tuple[int, int],
         partner: str,
         negotiator: SAONegotiator,
-        extra: Dict[str, Any] = None,
+        extra: dict[str, Any] | None = None,
+        consumer_starts: bool = True,
     ) -> bool:
         """
         Requests a negotiation
@@ -1755,6 +1813,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             partner: ID of the partner to negotiate with.
             negotiator: The negotiator to use for this negotiation (if the partner accepted to negotiate)
             extra: Extra information accessible through the negotiation annotation to the caller
+            consumer_starts: whether the consumer starts the negotiation
 
         Returns:
 
@@ -1762,7 +1821,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
 
         """
         agent = self.agents[agent_id]
-        is_buy = True
+        is_buy = consumer_starts
         if extra is None:
             extra = dict()
         # if is_buy:
@@ -1817,37 +1876,48 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
         #     self._registered_negs.add(tuple(sorted([partner, agent_id])))
         return result
 
-    def _make_issues(self, product):
+    def _make_issues(
+        self, product
+    ) -> tuple[tuple[int, int], tuple[int, int], tuple[int, int]]:
+        """Creates the negotiation agendas
+
+        Args:
+            product (int): The product to be negotiated about
+
+        Returns:
+            A tuple of minimum and maximum values for unit-price, time, and quantity in that order
+        """
+        price_of_product = (
+            self.trading_prices[product]
+            if self.publish_trading_prices
+            else self.catalog_prices[product]
+        )
+        if self.wide_price_range:
+            if product:
+                p = (
+                    self.trading_prices[product - 1]
+                    if self.publish_trading_prices
+                    else self.catalog_prices[product - 1]
+                )
+            else:
+                p = 0
+        else:
+            p = price_of_product
         unit_price = (
             max(
                 1,
-                int(
-                    1.0
-                    / self.price_multiplier
-                    * (
-                        self.trading_prices[product - 1]
-                        if self.publish_trading_prices
-                        else self.catalog_prices[product - 1]
-                    )
-                    if product
-                    else 0
-                ),
-            ),
-            int(
-                self.price_multiplier
-                * (
-                    self.trading_prices[product]
-                    if self.publish_trading_prices
-                    else self.catalog_prices[product]
-                )
+                int(p // self.price_multiplier),
             ),
+            int(self.price_multiplier * price_of_product),
         )
         time = (self.current_step, self.current_step)
         quantity = (1, self._max_n_lines)
         return unit_price, time, quantity
 
     def _make_negotiations(self):
-        def values(x: Union[int, Tuple[int, int]]):
+        consumer_starts = random.random() > 0.5
+
+        def values(x: int | tuple[int, int]):
             if not isinstance(x, Iterable):
                 return int(x), int(x)
             return int(min(x)), int(max(x))
@@ -1867,7 +1937,10 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
                 make_issue(values(time), name="time"),
                 make_issue(values(unit_price), name="unit_price"),
             ]
-            for aid in self.consumers[product]:
+            requesting_agents = (
+                self.consumers[product] if consumer_starts else self.suppliers[product]
+            )
+            for aid in requesting_agents:
                 if is_system_agent(aid) or isinstance(
                     self.agents[aid], OneShotSCML2020Adapter
                 ):
@@ -1881,6 +1954,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
                     controller=controllers[aid],
                     negotiators=None,
                     extra=None,
+                    consumer_starts=consumer_starts,
                 )
             # if not success:
             #     raise ValueError(
@@ -1894,7 +1968,7 @@ class SCML2020OneShotWorld(TimeInAgreementMixin, World):
             contract.executed_at = self.current_step
         return contracts
 
-    def get_private_state(self, agent: "Agent") -> dict:
+    def get_private_state(self, agent: Agent) -> dict:
         return agent.awi.state
 
     def _contract_record(self, contract):
