@@ -3,14 +3,15 @@ from collections import defaultdict
 from dataclasses import dataclass
 from random import random
 
-from negmas.common import AgentMechanismInterface
 from negmas.common import MechanismState
+from negmas.common import NegotiatorMechanismInterface
 from negmas.helpers import get_class
 from negmas.negotiators import Negotiator
+from negmas.outcomes import make_issue
+from negmas.preferences import LinearAdditiveUtilityFunction
 from negmas.situated import Breach
 from negmas.situated import Contract
 from negmas.situated import RenegotiationRequest
-from negmas.utilities import LinearUtilityAggregationFunction
 from numpy.random import dirichlet
 
 from .agent import SCML2019Agent
@@ -19,7 +20,11 @@ from .common import FinancialReport
 from .helpers import pos_gauss
 
 if True:
-    from typing import Dict, Any, List, Optional
+    from typing import Any
+    from typing import Dict
+    from typing import List
+    from typing import Optional
+
     from .common import Loan
 
 __all__ = ["Miner", "MiningProfile", "ReactiveMiner"]
@@ -86,11 +91,13 @@ class ReactiveMiner(Miner):
     def on_neg_request_rejected(self, req_id: str, by: Optional[List[str]]):
         pass
 
-    def on_neg_request_accepted(self, req_id: str, mechanism: AgentMechanismInterface):
+    def on_neg_request_accepted(
+        self, req_id: str, mechanism: NegotiatorMechanismInterface
+    ):
         pass
 
     def on_negotiation_success(
-        self, contract: Contract, mechanism: AgentMechanismInterface
+        self, contract: Contract, mechanism: NegotiatorMechanismInterface
     ) -> None:
         pass
 
@@ -143,7 +150,7 @@ class ReactiveMiner(Miner):
         self,
         partners: List[str],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
         state: MechanismState,
     ) -> None:
         # noinspection PyUnusedLocal
@@ -194,22 +201,21 @@ class ReactiveMiner(Miner):
         tau_t = pos_gauss(profile.tau_t, profile.cv)
         tau_q = pos_gauss(profile.tau_q, profile.cv)
 
-        ufun = LinearUtilityAggregationFunction(
-            issue_utilities={
-                "time": lambda x: x ** tau_t / beta_t,
-                "quantity": lambda x: x ** tau_q / beta_q,
-                "unit_price": lambda x: x ** tau_u / beta_u,
+        ufun = LinearAdditiveUtilityFunction(
+            values={
+                "time": lambda x: float(x) ** tau_t / beta_t if x else 0,
+                "quantity": lambda x: float(x) ** tau_q / beta_q if x else 0,
+                "unit_price": lambda x: float(x) ** tau_u / beta_u if x else 0,
             },
             weights={"time": alpha_t, "quantity": alpha_q, "unit_price": alpha_u},
+            issues=cfp.issues,
         )
         ufun.reserved_value = ufun(
-            {
-                "time": cfp.max_time,
-                "quantity": cfp.max_quantity,
-                "unit_price": cfp.money_resolution
-                if cfp.money_resolution is not None
-                else 0.0,
-            }
+            (
+                cfp.max_time,
+                cfp.max_quantity,
+                cfp.money_resolution if cfp.money_resolution is not None else 0.0,
+            )
         )
         # ufun = normalize(, outcomes=cfp.outcomes, infeasible_cutoff=-1)
         negotiator = self.negotiator_type(
