@@ -13,8 +13,6 @@ from typing import List
 from typing import Optional
 
 import numpy as np
-from negmas.java import to_dict
-from negmas.java import to_java
 
 from .common import NO_PRODUCTION
 from .common import Factory
@@ -1499,9 +1497,6 @@ class FastFactorySimulator(FactorySimulator):
 
         self.fix_before(t)
 
-    class Java:
-        implements = ["jnegmas.apps.scml.simulators.FactorySimulator"]
-
 
 @contextmanager
 def transaction(simulator):
@@ -1518,168 +1513,3 @@ def temporary_transaction(simulator):
     yield _bookmark
     simulator.rollback(_bookmark)
     simulator.delete_bookmark(_bookmark)
-
-
-class _ShadowFactorySimulator:
-    """An FactorySimulator As seen by JNegMAS.
-
-        This is an object that is not visible to python code. It is not directly called from python ever. It is only called
-        from a corresponding Java object to represent an internal python object. Because of he way py4j works, we cannot
-        just use dunders to implement this kind of object in general. We will have to implement each such class
-        independently.
-
-        This kind of classes will always have an internal Java class implementing a Java interface in Jnegmas that starts
-        with Py.
-
-    """
-
-    def __init__(self, simulator: FactorySimulator):
-        self.shadow = simulator
-
-    def maxStorage(self) -> int:
-        return to_java(self.shadow.max_storage)
-
-    def nSteps(self) -> int:
-        return to_java(self.shadow.n_steps)
-
-    def initialWallet(self) -> float:
-        return to_java(self.shadow.initial_wallet)
-
-    def initialStorage(self) -> List[int]:
-        return to_java(self.shadow.initial_storage.tolist())
-
-    def nLines(self) -> int:
-        return to_java(self.shadow.n_lines)
-
-    def finalBalance(self) -> float:
-        return to_java(self.shadow.final_balance)
-
-    def walletTo(self, t: int) -> List[float]:
-        return to_java(self.shadow.wallet_to(t).tolist())
-
-    def walletAt(self, t: int) -> float:
-        return to_java(self.shadow.wallet_at(t))
-
-    def storageTo(self, t: int) -> List[int]:
-        return to_java(self.shadow.storage_to(t).tolinst())
-
-    def storageAt(self, t: int) -> int:
-        return to_java(self.shadow.storage_at(t))
-
-    def line_schedules_to(self, t: int) -> List[List[int]]:
-        return to_java(self.shadow.line_schedules_to(t).tolist())
-
-    def line_schedules_at(self, t: int) -> List[int]:
-        return to_java(self.shadow.line_schedules_at(t).tolist())
-
-    def totalStorageTo(self, t: int) -> List[int]:
-        return to_java(self.shadow.total_storage_to(t).tolist())
-
-    def totalStorageAt(self, t: int) -> int:
-        return to_java(self.shadow.total_storage_at(t))
-
-    def reservedStorageTo(self, t: int) -> List[int]:
-        return to_java(self.shadow.reserved_storage_to(t).tolist())
-
-    def reservedStorageAt(self, t: int) -> int:
-        return to_java(self.shadow.reserved_storage_at(t))
-
-    def availableStorageTo(self, t: int) -> List[int]:
-        return to_java(self.shadow.available_storage_to(t).tolist())
-
-    def availableStorageAt(self, t: int) -> int:
-        return to_java(self.shadow.available_storage_at(t))
-
-    def loansTo(self, t: int) -> List[float]:
-        return to_java(self.shadow.loans_to(t).tolist())
-
-    def loansAt(self, t: int) -> float:
-        return to_java(self.shadow.loans_at(t))
-
-    def balanceTo(self, t: int) -> List[float]:
-        return to_java(self.shadow.balance_to(t).tolist())
-
-    def balanceAt(self, t: int) -> float:
-        return to_java(self.shadow.balance_at(t))
-
-    def fixedBefore(self) -> int:
-        return to_java(self.shadow.fixed_before)
-
-    def setState(
-        self,
-        t: int,
-        storage: List[int],
-        wallet: float,
-        loans: float,
-        lineSchedules: List[int],
-    ) -> None:
-        self.shadow.set_state(
-            t, storage=storage, wallet=wallet, loans=loans, line_schedules=lineSchedules
-        )
-
-    def addLoan(self, total: float, t: int) -> bool:
-        return self.shadow.add_loan(total=total, t=t)
-
-    def receive(self, payment, t):
-        return self.shadow.receive(payment, t)
-
-    def pay(self, payment, t, ignoreMoneyShortage):
-        return self.shadow.pay(payment, t, ignoreMoneyShortage)
-
-    def transportTo(
-        self, product, quantity, t, ignoreInventoryShortage, ignoreSpaceShortage
-    ):
-        return self.shadow.transport_to(
-            product, quantity, t, ignoreInventoryShortage, ignoreSpaceShortage
-        )
-
-    def buy(
-        self, product, quantity, price, t, ignoreMoneyShortage, ignoreSpaceShortage
-    ):
-        return self.shadow.buy(
-            product, quantity, price, t, ignoreMoneyShortage, ignoreSpaceShortage
-        )
-
-    def sell(
-        self, product, quantity, price, t, ignoreMoneyShortage, ignoreInventoryShortage
-    ):
-        return self.shadow.sell(
-            product, quantity, price, t, ignoreMoneyShortage, ignoreInventoryShortage
-        )
-
-    def schedule(
-        self,
-        job,
-        ignoreInventoryShortage,
-        ignoreMoneyShortage,
-        ignoreSpaceShortage,
-        override,
-    ):
-        return self.shadow.schedule(
-            job,
-            ignoreInventoryShortage,
-            ignoreMoneyShortage,
-            ignoreSpaceShortage,
-            override,
-        )
-
-    def reserve(self, product, quantity, t):
-        return self.reserve(product, quantity, t)
-
-    def fixBefore(self, t):
-        return self.shadow.fix_before(t)
-
-    def bookmark(self):
-        return self.shadow.bookmark()
-
-    def rollback(self, bookmarkId):
-        return self.shadow.rollback(bookmarkId)
-
-    def deleteBookmark(self, bookmarkId):
-        return self.shadow.delete_bookmark(bookmarkId)
-
-    def to_java(self):
-        return to_dict(self.shadow)
-
-    class Java:
-        implements = ["jengmas.apps.scml.simulators.FactorySimulator"]
