@@ -1,27 +1,32 @@
 """Implements the world class for the SCML2020 world """
+from typing import Any
+from typing import Dict
+from typing import List
+from typing import Optional
+from typing import Tuple
+from typing import Union
+
+import numpy as np
 from negmas import Agent
-from negmas import AgentMechanismInterface
 from negmas import Breach
 from negmas import Contract
 from negmas import Issue
 from negmas import MechanismState
 from negmas import Negotiator
+from negmas import NegotiatorMechanismInterface
 from negmas import RenegotiationRequest
-from typing import Dict, Union, Any, Optional, List, Tuple
-import numpy as np
+from negmas import make_issue
+from negmas.helpers import get_full_type_name
+from negmas.helpers import instantiate
 from negmas.situated import Adapter
-from negmas.helpers import instantiate, get_full_type_name
 
-
-from ..oneshot.ufun import OneShotUFun
 from ..oneshot.agent import OneShotAgent
-from ..oneshot.common import OneShotState, OneShotProfile
+from ..oneshot.common import OneShotProfile
+from ..oneshot.common import OneShotState
 from ..oneshot.mixins import OneShotUFunCreatorMixin
-
-from .components.trading import MarketAwareTradePredictionStrategy
 from .components.production import DemandDrivenProductionStrategy
 from .components.signing import SignAll
-from .common import TIME, QUANTITY, UNIT_PRICE
+from .components.trading import MarketAwareTradePredictionStrategy
 
 __all__ = [
     "SCML2020Agent",
@@ -69,7 +74,7 @@ class SCML2020Agent(Agent):
         partners: List[str],
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
         role: Optional[str],
         req_id: Optional[str],
     ) -> Optional[Negotiator]:
@@ -92,7 +97,9 @@ class SCML2020Agent(Agent):
     def on_neg_request_rejected(self, req_id: str, by: Optional[List[str]]):
         pass
 
-    def on_neg_request_accepted(self, req_id: str, mechanism: AgentMechanismInterface):
+    def on_neg_request_accepted(
+        self, req_id: str, mechanism: NegotiatorMechanismInterface
+    ):
         pass
 
     @property
@@ -104,13 +111,13 @@ class SCML2020Agent(Agent):
         self,
         partners: List[str],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
         state: MechanismState,
     ) -> None:
         """Called whenever a negotiation ends without agreement"""
 
     def on_negotiation_success(
-        self, contract: Contract, mechanism: AgentMechanismInterface
+        self, contract: Contract, mechanism: NegotiatorMechanismInterface
     ) -> None:
         """Called whenever a negotiation ends with agreement"""
 
@@ -153,7 +160,7 @@ class SCML2020Agent(Agent):
         initiator: str,
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
     ) -> Optional[Negotiator]:
         """
         Called whenever another agent requests a negotiation with this agent.
@@ -162,7 +169,7 @@ class SCML2020Agent(Agent):
             initiator: The ID of the agent that requested this negotiation
             issues: Negotiation issues
             annotation: Annotation attached with this negotiation
-            mechanism: The `AgentMechanismInterface` interface to the mechanism to be used for this negotiation.
+            mechanism: The `NegotiatorMechanismInterface` interface to the mechanism to be used for this negotiation.
 
         Returns:
             None to reject the negotiation, otherwise a negotiator
@@ -225,7 +232,7 @@ class _SystemAgent(SCML2020Agent):
         initiator: str,
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
     ) -> Optional[Negotiator]:
         pass
 
@@ -234,6 +241,7 @@ class _SystemAgent(SCML2020Agent):
 
     def before_step(self):
         pass
+
     def init(self):
         pass
 
@@ -241,13 +249,13 @@ class _SystemAgent(SCML2020Agent):
         self,
         partners: List[str],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
         state: MechanismState,
     ) -> None:
         pass
 
     def on_negotiation_success(
-        self, contract: Contract, mechanism: AgentMechanismInterface
+        self, contract: Contract, mechanism: NegotiatorMechanismInterface
     ) -> None:
         pass
 
@@ -405,9 +413,9 @@ class AWIHelper:
         else:
             u, t, q = self._owner._make_issues(self.my_input_product)
             issues = [
-                Issue(values=q, name="quantity"),
-                Issue(values=t, name="time"),
-                Issue(values=u, name="unit_price"),
+                make_issue(values=q, name="quantity"),
+                make_issue(values=t, name="time"),
+                make_issue(values=u, name="unit_price"),
             ]
         return issues
 
@@ -418,9 +426,9 @@ class AWIHelper:
         else:
             u, t, q = self._owner._make_issues(self.my_output_product)
             issues = [
-                Issue(values=q, name="quantity"),
-                Issue(values=t, name="time"),
-                Issue(values=u, name="unit_price"),
+                make_issue(values=q, name="quantity"),
+                make_issue(values=t, name="time"),
+                make_issue(values=u, name="unit_price"),
             ]
         return issues
 
@@ -449,8 +457,10 @@ class AWIHelper:
         x = np.zeros_like(self._world.exogenous_contracts_summary)
         for i in range(0, n_products):
             x[i, 0 : n_steps - i, :] = y[i, i:n_steps, :]
-        summary = [(int(x[i, self.current_step, 0]), int(x[i, self.current_step, 1]))
-        for i in range(n_products)]
+        summary = [
+            (int(x[i, self.current_step, 0]), int(x[i, self.current_step, 1]))
+            for i in range(n_products)
+        ]
         return summary
 
     # Everything else
@@ -465,7 +475,7 @@ class OneShotAdapter(
     MarketAwareTradePredictionStrategy,
     SCML2020Agent,
     Adapter,
-    OneShotUFunCreatorMixin
+    OneShotUFunCreatorMixin,
 ):
     """
     An adapter allowing agents developed for SCML-OneShot to run in
@@ -533,6 +543,7 @@ class OneShotAdapter(
                 )
             ),
         )
+        unit_price = (int(unit_price[0]), int(max(unit_price)))
         t = self.awi.current_step + (self.awi.my_output_product == product)
         time = (t, t)
         quantity = (1, self.awi.n_lines)
