@@ -11,15 +11,18 @@ from typing import Tuple
 from typing import Union
 
 import numpy as np
-from negmas import AgentMechanismInterface
 from negmas import AspirationNegotiator
 from negmas import Contract
 from negmas import Issue
+from negmas import LinearUtilityFunction
 from negmas import Negotiator
+from negmas import NegotiatorMechanismInterface
 from negmas import SAONegotiator
 from negmas import UtilityFunction
+from negmas import make_issue
 from negmas.helpers import get_class
 from negmas.helpers import instantiate
+from negmas.outcomes.issue_ops import enumerate_issues
 
 from scml.scml2020 import AWI
 from scml.scml2020.common import TIME
@@ -321,7 +324,7 @@ class NegotiationManager:
         initiator: str,
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
     ) -> Optional[Negotiator]:
         raise ValueError("You must implement respond_to_negotiation_request")
 
@@ -455,7 +458,7 @@ class StepNegotiationManager(MeanERPStrategy, NegotiationManager):
         initiator: str,
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
     ) -> Optional[Negotiator]:
 
         # find negotiation parameters
@@ -472,7 +475,7 @@ class StepNegotiationManager(MeanERPStrategy, NegotiationManager):
             return None
         # self.awi.loginfo_agent(
         #     f"Accepting request from {initiator}: {[str(_) for _ in mechanism.issues]} "
-        #     f"({Issue.num_outcomes(mechanism.issues)})"
+        #     f"({num_outcomes(mechanism.issues)})"
         # )
         # create a controller for the time-step if one does not exist or use the one already running
         if controller_info.controller is None:
@@ -672,9 +675,9 @@ class IndependentNegotiationsManager(NegotiationManager):
         # negotiate with all suppliers of the input product I need to produce
 
         issues = [
-            Issue((int(qvalues[0]), int(qvalues[1])), name="quantity"),
-            Issue((int(tvalues[0]), int(tvalues[1])), name="time"),
-            Issue((int(uvalues[0]), int(uvalues[1])), name="uvalues"),
+            make_issue((int(qvalues[0]), int(qvalues[1])), name="quantity"),
+            make_issue((int(tvalues[0]), int(tvalues[1])), name="time"),
+            make_issue((int(uvalues[0]), int(uvalues[1])), name="unit_price"),
         ]
 
         for partner in partners:
@@ -693,26 +696,24 @@ class IndependentNegotiationsManager(NegotiationManager):
         initiator: str,
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
     ) -> Optional[Negotiator]:
         return self.negotiator(
             annotation["seller"] == self.id, issues=issues, partner=initiator
         )
 
-    @abstractmethod
     def create_ufun(
         self, is_seller: bool, issues=None, outcomes=None
     ) -> UtilityFunction:
         """Creates a utility function"""
+        if is_seller:
+            return LinearUtilityFunction((1, 1, 10), issues=issues, outcomes=outcomes)
+        return LinearUtilityFunction((1, -1, -10), issues=issues, outcomes=outcomes)
 
     def negotiator(
         self, is_seller: bool, issues=None, outcomes=None, partner=None
     ) -> SAONegotiator:
         """Creates a negotiator"""
-        if outcomes is None and (
-            issues is None or not Issue.enumerate(issues, astype=tuple)
-        ):
-            return None
         params = self.negotiator_params
         params["ufun"] = self.create_ufun(
             is_seller=is_seller, outcomes=outcomes, issues=issues
@@ -858,7 +859,7 @@ class MovingRangeNegotiationManager:
         initiator: str,
         issues: List[Issue],
         annotation: Dict[str, Any],
-        mechanism: AgentMechanismInterface,
+        mechanism: NegotiatorMechanismInterface,
     ) -> Optional[Negotiator]:
         if not (
             issues[TIME].min_value < self._current_end
