@@ -38,6 +38,8 @@ from scml.oneshot.common import UNIT_PRICE
 from scml.oneshot.ufun import OneShotUFun
 from scml.scml2020 import is_system_agent
 
+from .switches import SCML_ON_GITHUB
+
 random.seed(0)
 
 COMPACT = True
@@ -205,6 +207,7 @@ def test_can_run_with_a_multiple_agent_types(agent_types, n_processes):
     save_stats(world, world.log_folder)
 
 
+@pytest.mark.skipif(SCML_ON_GITHUB, reason="Known to timeout on CI")
 @given(n_processes=st.integers(2, 2))
 @settings(deadline=300_000, max_examples=20)
 def test_something_happens_with_random_agents(n_processes):
@@ -676,20 +679,21 @@ def test_builtin_aspiration():
     world.run()
 
 
-# @given(
-#     atype=st.lists(
-#         st.sampled_from(std_types + types), unique=True, min_size=2, max_size=6  # type: ignore
-#     )
-# )
-# @settings(deadline=900_000, max_examples=10)
-# def test_adapter(atype):
-#     world = SCML2020OneShotWorld(
-#         **SCML2020OneShotWorld.generate(agent_types=atype, n_steps=10),
-#         construct_graphs=False,
-#         compact=True,
-#         no_logs=True,
-#     )
-#     world.run()
+@pytest.mark.skipif(SCML_ON_GITHUB, reason="Known to timeout on CI")
+@given(
+    atype=st.lists(
+        st.sampled_from(std_types + types), unique=True, min_size=2, max_size=6  # type: ignore
+    )
+)
+@settings(deadline=900_000, max_examples=10)
+def test_adapter(atype):
+    world = SCML2020OneShotWorld(
+        **SCML2020OneShotWorld.generate(agent_types=atype, n_steps=10),
+        construct_graphs=False,
+        compact=True,
+        no_logs=True,
+    )
+    world.run()
 
 
 class MyIndNeg(OneShotIndNegotiatorsAgent):
